from Products.ZPatterns.Agents import Agent, defaultConstructors, GTMixin
from Products.ZPatterns.Proxy import ProxyManager
import sys
from zLOG import LOG, WARNING
from Products.ZPatterns.Expressions import *
from string import join

class CatalogTrigger(GTMixin, Agent, ProxyManager):
    """ Catalog/Recatalog/Uncatalog objects on add/change/delete 
    
    This trigger can be used as a replacement for CatalogAwareness.
    """
    
    meta_type = "Cataloging Trigger"
    manage_options = Agent.manage_options + ProxyManager.manage_options
    handle_set_for = ()
    applicable_events = event_kinds = 'ADD', 'CHANGE', 'DELETE'

    
    _properties=(
        {'id':'catalog_expr', 'type': 'string', 'mode': 'w'},
        {'id':'handle_change','type': 'boolean', 'mode': 'w'},
    )
    
    propertyLabels = {
        'catalog_expr' : "Expression:",
        'handle_change' : 'Recatalog when objects change:',
    }
    
    def __init__(self, id, title=''):
        Agent.__init__(self, id, title)
        self.__dict__['catalog_expr'] = "Catalog"
        self.__dict__['handle_change'] = 1
     
     
    def _propertiesChanged(self):
        if self.catalog_expr:
            self._catalogex = Expression(self.catalog_expr)
        else:
            self._catalogex = None
            
        
    # Event handlers, overriding Agent's defaults
    def _getMementoFor(self,client):
        return client.__dict__
        
    def _objectChanged(self,client,memento):
        if self.handle_change:
            self.recatalog_object(client)

    def _objectDeleted(self,client,memento):
        self.uncatalog_object(client)

    def _objectAdded(self,client):
        self.catalog_object(client)
            
    # Cataloging methods
    def catalog_object(self, object):
        """A common method to allow Findables to index themselves."""
        try:
            self._eval_catalogex(object, catalog=1)
        except:
            LOG('DataSkinAddons',WARNING,('Cataloging failed for %s' % object),
                '', sys.exc_info(), 1)        

    def uncatalog_object(self, object):
        """A common method to allow Findables to unindex themselves."""
        try:
            self._eval_catalogex(object, uncatalog=1)
        except:
            LOG('DataSkinAddons',WARNING,('Cataloging failed for %s' % object),
                '', sys.exc_info(), 1)

    def recatalog_object(self, object):
        """ Suprisingly useful """
        try:
            self._eval_catalogex(object, recatalog=1)
        except:
            LOG('DataSkinAddons',WARNING,('Recataloging failed for %s' % object),
                '', sys.exc_info(), 1)

    def _eval_catalogex(self, object, catalog=0, uncatalog=0, recatalog=0):
        # in the expression, use "self" to mean "the dataskin object being cataloged"
        # use "this" to mean "this catalog trigger".
        if self._catalogex is not None:
            md = NamespaceStack()
            i = InstanceDict(self, md)
            md._push(i)
            md._push({'self':object, 'this':self})
            getSecurityManager().addContext(self)
            try:
                if uncatalog or recatalog:
                    self._catalogex.eval(md).uncatalog_object(url(object))
                if catalog or recatalog:
                    self._catalogex.eval(md).catalog_object(object, url(object))                            
            finally:
                getSecurityManager().removeContext(self)
                md._pop(2)
            

def url(object):
    """Return a Traversable.getPhysicalPath() based url for an object."""
    return join(object.getPhysicalPath(), '/')

    
def initialize(context):
    context.registerPlugInClass(
        CatalogTrigger,
        permission = 'Add Generic Triggers',
        constructors = defaultConstructors(CatalogTrigger,globals()),
        icon = 'www/catalogtrigger.gif'
    )    
    
